@@ -1,34 +1,5 @@
-/*
- * Copyright 2015-2020, Intel Corporation
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in
- *       the documentation and/or other materials provided with the
- *       distribution.
- *
- *     * Neither the name of the copyright holder nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// SPDX-License-Identifier: BSD-3-Clause
+/* Copyright 2015-2020, Intel Corporation */
 
 /*
  * tx.c -- transactions implementation
@@ -49,6 +20,7 @@
 struct tx_data {
 	PMDK_SLIST_ENTRY(tx_data) tx_entry;
 	jmp_buf env;
+	enum pobj_tx_failure_behavior failure_behavior;
 };
 
 struct tx {
@@ -621,7 +593,7 @@ tx_alloc_common(struct tx *tx, size_t size, type_num_t type_num,
 	PMEMoid retoid = OID_NULL;
 	retoid.off = action->heap.offset;
 	retoid.pool_uuid_lo = pop->uuid_lo;
-	size = palloc_usable_size(&pop->heap, retoid.off);
+	size = action->heap.usable_size;
 
 	const struct tx_range_def r = {retoid.off, size, args.flags};
 	if (tx_lane_ranges_insert_def(pop, tx, &r) != 0)
@@ -748,6 +720,8 @@ pmemobj_tx_begin(PMEMobjpool *pop, jmp_buf env, ...)
 	int err = 0;
 	struct tx *tx = get_tx();
 
+	enum pobj_tx_failure_behavior failure_behavior = POBJ_TX_FAILURE_ABORT;
+
 	if (tx->stage == TX_STAGE_WORK) {
 		ASSERTne(tx->lane, NULL);
 		if (tx->pop != pop) {
@@ -755,6 +729,10 @@ pmemobj_tx_begin(PMEMobjpool *pop, jmp_buf env, ...)
 			return obj_tx_fail_err(EINVAL, 0);
 		}
 
+		/* inherits this value from the parent transaction */
+		struct tx_data *txd = PMDK_SLIST_FIRST(&tx->tx_entries);
+		failure_behavior = txd->failure_behavior;
+
 		VALGRIND_START_TX;
 	} else if (tx->stage == TX_STAGE_NONE) {
 		VALGRIND_START_TX;
@@ -793,6 +771,8 @@ pmemobj_tx_begin(PMEMobjpool *pop, jmp_buf env, ...)
 	else
 		memset(txd->env, 0, sizeof(jmp_buf));
 
+	txd->failure_behavior = failure_behavior;
+
 	PMDK_SLIST_INSERT_HEAD(&tx->tx_entries, txd, tx_entry);
 
 	tx->stage = TX_STAGE_WORK;
@@ -842,6 +822,20 @@ err_abort:
 	return err;
 }
 
+/*
+ * tx_abort_on_failure_flag -- (internal) return 0 or POBJ_FLAG_TX_NO_ABORT
+ * based on transaction setting
+ */
+static uint64_t
+tx_abort_on_failure_flag(struct tx *tx)
+{
+	struct tx_data *txd = PMDK_SLIST_FIRST(&tx->tx_entries);
+
+	if (txd->failure_behavior == POBJ_TX_FAILURE_RETURN)
+		return POBJ_FLAG_TX_NO_ABORT;
+	return 0;
+}
+
 /*
  * pmemobj_tx_xlock -- get lane from pool and add lock to transaction,
  * with no_abort option
@@ -849,16 +843,18 @@ err_abort:
 int
 pmemobj_tx_xlock(enum pobj_tx_param type, void *lockp, uint64_t flags)
 {
+	struct tx *tx = get_tx();
+	ASSERT_IN_TX(tx);
+	ASSERT_TX_STAGE_WORK(tx);
+
+	flags |= tx_abort_on_failure_flag(tx);
+
 	if (flags & ~POBJ_XLOCK_VALID_FLAGS) {
 		ERR("unknown flags 0x%" PRIx64,
 				flags & ~POBJ_XLOCK_VALID_FLAGS);
 		return obj_tx_fail_err(EINVAL, flags);
 	}
 
-	struct tx *tx = get_tx();
-	ASSERT_IN_TX(tx);
-	ASSERT_TX_STAGE_WORK(tx);
-
 	int ret = add_to_tx_and_lock(tx, type, lockp);
 	if (ret)
 		return obj_tx_fail_err(ret, flags);
@@ -1127,7 +1123,7 @@ static void
 vg_verify_initialized(PMEMobjpool *pop, const struct tx_range_def *def)
 {
 #if VG_MEMCHECK_ENABLED
-	if (!On_valgrind)
+	if (!On_memcheck)
 		return;
 
 	VALGRIND_DO_DISABLE_ERROR_REPORTING;
@@ -1388,22 +1384,24 @@ pmemobj_tx_add_range_direct(const void *ptr, size_t size)
 	ASSERT_IN_TX(tx);
 	ASSERT_TX_STAGE_WORK(tx);
 
-	PMEMobjpool *pop = tx->pop;
+	int ret;
 
-	if (!OBJ_PTR_FROM_POOL(pop, ptr)) {
+	uint64_t flags = tx_abort_on_failure_flag(tx);
+
+	if (!OBJ_PTR_FROM_POOL(tx->pop, ptr)) {
 		ERR("object outside of pool");
-		int ret = obj_tx_fail_err(EINVAL, 0);
+		ret = obj_tx_fail_err(EINVAL, flags);
 		PMEMOBJ_API_END();
 		return ret;
 	}
 
 	struct tx_range_def args = {
-		.offset = (uint64_t)((char *)ptr - (char *)pop),
+		.offset = (uint64_t)((char *)ptr - (char *)tx->pop),
 		.size = size,
-		.flags = 0,
+		.flags = flags,
 	};
 
-	int ret = pmemobj_tx_add_common(tx, &args);
+	ret = pmemobj_tx_add_common(tx, &args);
 
 	PMEMOBJ_API_END();
 	return ret;
@@ -1417,14 +1415,17 @@ int
 pmemobj_tx_xadd_range_direct(const void *ptr, size_t size, uint64_t flags)
 {
 	LOG(3, NULL);
+
+	PMEMOBJ_API_START();
 	struct tx *tx = get_tx();
 
 	ASSERT_IN_TX(tx);
 	ASSERT_TX_STAGE_WORK(tx);
 
-	PMEMOBJ_API_START();
 	int ret;
 
+	flags |= tx_abort_on_failure_flag(tx);
+
 	if (flags & ~POBJ_XADD_VALID_FLAGS) {
 		ERR("unknown flags 0x%" PRIx64, flags
 			& ~POBJ_XADD_VALID_FLAGS);
@@ -1466,9 +1467,13 @@ pmemobj_tx_add_range(PMEMoid oid, uint64_t hoff, size_t size)
 	ASSERT_IN_TX(tx);
 	ASSERT_TX_STAGE_WORK(tx);
 
+	int ret;
+
+	uint64_t flags = tx_abort_on_failure_flag(tx);
+
 	if (oid.pool_uuid_lo != tx->pop->uuid_lo) {
 		ERR("invalid pool uuid");
-		int ret = obj_tx_fail_err(EINVAL, 0);
+		ret = obj_tx_fail_err(EINVAL, flags);
 		PMEMOBJ_API_END();
 		return ret;
 	}
@@ -1477,10 +1482,10 @@ pmemobj_tx_add_range(PMEMoid oid, uint64_t hoff, size_t size)
 	struct tx_range_def args = {
 		.offset = oid.off + hoff,
 		.size = size,
-		.flags = 0,
+		.flags = flags,
 	};
 
-	int ret = pmemobj_tx_add_common(tx, &args);
+	ret = pmemobj_tx_add_common(tx, &args);
 
 	PMEMOBJ_API_END();
 	return ret;
@@ -1502,6 +1507,8 @@ pmemobj_tx_xadd_range(PMEMoid oid, uint64_t hoff, size_t size, uint64_t flags)
 
 	int ret;
 
+	flags |= tx_abort_on_failure_flag(tx);
+
 	if (flags & ~POBJ_XADD_VALID_FLAGS) {
 		ERR("unknown flags 0x%" PRIx64, flags
 			& ~POBJ_XADD_VALID_FLAGS);
@@ -1525,6 +1532,7 @@ pmemobj_tx_xadd_range(PMEMoid oid, uint64_t hoff, size_t size, uint64_t flags)
 	};
 
 	ret = pmemobj_tx_add_common(tx, &args);
+
 	PMEMOBJ_API_END();
 	return ret;
 }
@@ -1543,16 +1551,18 @@ pmemobj_tx_alloc(size_t size, uint64_t type_num)
 	ASSERT_IN_TX(tx);
 	ASSERT_TX_STAGE_WORK(tx);
 
+	uint64_t flags = tx_abort_on_failure_flag(tx);
+
 	PMEMoid oid;
 	if (size == 0) {
 		ERR("allocation with size 0");
-		oid = obj_tx_fail_null(EINVAL, 0);
+		oid = obj_tx_fail_null(EINVAL, flags);
 		PMEMOBJ_API_END();
 		return oid;
 	}
 
 	oid = tx_alloc_common(tx, size, (type_num_t)type_num,
-			constructor_tx_alloc, ALLOC_ARGS(0));
+			constructor_tx_alloc, ALLOC_ARGS(flags));
 
 	PMEMOBJ_API_END();
 	return oid;
@@ -1570,17 +1580,20 @@ pmemobj_tx_zalloc(size_t size, uint64_t type_num)
 	ASSERT_IN_TX(tx);
 	ASSERT_TX_STAGE_WORK(tx);
 
+	uint64_t flags = POBJ_FLAG_ZERO;
+	flags |= tx_abort_on_failure_flag(tx);
+
 	PMEMOBJ_API_START();
 	PMEMoid oid;
 	if (size == 0) {
 		ERR("allocation with size 0");
-		oid = obj_tx_fail_null(EINVAL, 0);
+		oid = obj_tx_fail_null(EINVAL, flags);
 		PMEMOBJ_API_END();
 		return oid;
 	}
 
 	oid = tx_alloc_common(tx, size, (type_num_t)type_num,
-			constructor_tx_alloc, ALLOC_ARGS(POBJ_FLAG_ZERO));
+			constructor_tx_alloc, ALLOC_ARGS(flags));
 
 	PMEMOBJ_API_END();
 	return oid;
@@ -1597,6 +1610,9 @@ pmemobj_tx_xalloc(size_t size, uint64_t type_num, uint64_t flags)
 
 	ASSERT_IN_TX(tx);
 	ASSERT_TX_STAGE_WORK(tx);
+
+	flags |= tx_abort_on_failure_flag(tx);
+
 	PMEMOBJ_API_START();
 
 	PMEMoid oid;
@@ -1669,17 +1685,19 @@ pmemobj_tx_xstrdup(const char *s, uint64_t type_num, uint64_t flags)
 {
 	LOG(3, NULL);
 
+	struct tx *tx = get_tx();
+
+	ASSERT_IN_TX(tx);
+	ASSERT_TX_STAGE_WORK(tx);
+
+	flags |= tx_abort_on_failure_flag(tx);
+
 	if (flags & ~POBJ_TX_XALLOC_VALID_FLAGS) {
 		ERR("unknown flags 0x%" PRIx64,
 				flags & ~POBJ_TX_XALLOC_VALID_FLAGS);
 		return obj_tx_fail_null(EINVAL, flags);
 	}
 
-	struct tx *tx = get_tx();
-
-	ASSERT_IN_TX(tx);
-	ASSERT_TX_STAGE_WORK(tx);
-
 	PMEMOBJ_API_START();
 	PMEMoid oid;
 	if (NULL == s) {
@@ -1725,17 +1743,19 @@ pmemobj_tx_xwcsdup(const wchar_t *s, uint64_t type_num, uint64_t flags)
 {
 	LOG(3, NULL);
 
+	struct tx *tx = get_tx();
+
+	ASSERT_IN_TX(tx);
+	ASSERT_TX_STAGE_WORK(tx);
+
+	flags |= tx_abort_on_failure_flag(tx);
+
 	if (flags & ~POBJ_TX_XALLOC_VALID_FLAGS) {
 		ERR("unknown flags 0x%" PRIx64,
 				flags & ~POBJ_TX_XALLOC_VALID_FLAGS);
 		return obj_tx_fail_null(EINVAL, flags);
 	}
 
-	struct tx *tx = get_tx();
-
-	ASSERT_IN_TX(tx);
-	ASSERT_TX_STAGE_WORK(tx);
-
 	PMEMOBJ_API_START();
 	PMEMoid oid;
 	if (NULL == s) {
@@ -1782,17 +1802,19 @@ pmemobj_tx_xfree(PMEMoid oid, uint64_t flags)
 {
 	LOG(3, NULL);
 
+	struct tx *tx = get_tx();
+
+	ASSERT_IN_TX(tx);
+	ASSERT_TX_STAGE_WORK(tx);
+
+	flags |= tx_abort_on_failure_flag(tx);
+
 	if (flags & ~POBJ_XFREE_VALID_FLAGS) {
 		ERR("unknown flags 0x%" PRIx64,
 				flags & ~POBJ_XFREE_VALID_FLAGS);
 		return obj_tx_fail_err(EINVAL, flags);
 	}
 
-	struct tx *tx = get_tx();
-
-	ASSERT_IN_TX(tx);
-	ASSERT_TX_STAGE_WORK(tx);
-
 	if (OBJ_OID_IS_NULL(oid))
 		return 0;
 
@@ -1863,14 +1885,19 @@ pmemobj_tx_free(PMEMoid oid)
 int
 pmemobj_tx_xpublish(struct pobj_action *actv, size_t actvcnt, uint64_t flags)
 {
+	struct tx *tx = get_tx();
+
+	ASSERT_IN_TX(tx);
+	ASSERT_TX_STAGE_WORK(tx);
+
+	flags |= tx_abort_on_failure_flag(tx);
+
 	if (flags & ~POBJ_XPUBLISH_VALID_FLAGS) {
 		ERR("unknown flags 0x%" PRIx64,
 				flags & ~POBJ_XPUBLISH_VALID_FLAGS);
 		return obj_tx_fail_err(EINVAL, flags);
 	}
 
-	struct tx *tx = get_tx();
-	ASSERT_TX_STAGE_WORK(tx);
 	PMEMOBJ_API_START();
 
 	if (tx_action_reserve(tx, actvcnt) != 0) {
@@ -1903,14 +1930,19 @@ int
 pmemobj_tx_xlog_append_buffer(enum pobj_log_type type, void *addr, size_t size,
 		uint64_t flags)
 {
+	struct tx *tx = get_tx();
+
+	ASSERT_IN_TX(tx);
+	ASSERT_TX_STAGE_WORK(tx);
+
+	flags |= tx_abort_on_failure_flag(tx);
+
 	if (flags & ~POBJ_XLOG_APPEND_BUFFER_VALID_FLAGS) {
 		ERR("unknown flags 0x%" PRIx64,
 				flags & ~POBJ_XLOG_APPEND_BUFFER_VALID_FLAGS);
 		return obj_tx_fail_err(EINVAL, flags);
 	}
 
-	struct tx *tx = get_tx();
-	ASSERT_TX_STAGE_WORK(tx);
 	PMEMOBJ_API_START();
 	int err;
 
@@ -2082,6 +2114,44 @@ pmemobj_tx_get_user_data(void)
 	return tx->user_data;
 }
 
+/*
+ * pmemobj_tx_set_failure_behavior -- enables or disables automatic transaction
+ * abort in case of an error
+ */
+void
+pmemobj_tx_set_failure_behavior(enum pobj_tx_failure_behavior behavior)
+{
+	LOG(3, "behavior %d", behavior);
+
+	struct tx *tx = get_tx();
+
+	ASSERT_IN_TX(tx);
+	ASSERT_TX_STAGE_WORK(tx);
+
+	struct tx_data *txd = PMDK_SLIST_FIRST(&tx->tx_entries);
+
+	txd->failure_behavior = behavior;
+}
+
+/*
+ * pmemobj_tx_get_failure_behavior -- returns enum specifying failure event
+ * for the current transaction.
+ */
+enum pobj_tx_failure_behavior
+pmemobj_tx_get_failure_behavior(void)
+{
+	LOG(3, NULL);
+
+	struct tx *tx = get_tx();
+
+	ASSERT_IN_TX(tx);
+	ASSERT_TX_STAGE_WORK(tx);
+
+	struct tx_data *txd = PMDK_SLIST_FIRST(&tx->tx_entries);
+
+	return txd->failure_behavior;
+}
+
 /*
  * CTL_READ_HANDLER(size) -- gets the cache size transaction parameter
  */
